Package: gpciImpSam
Type: Package
Title: Importance Sampling Estimation of Generalized Process Capability
        Indices
Version: 0.1.0
Authors@R: c(
    person("Shikhar", "Tyagi",
           email = "shikhar1093tyagi@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-1606-0844"))
    )
Description: Provides a comprehensive generalized framework for parameter estimation 
    and Generalized Process Capability Indices (GPCIs) under uncensored data using 
    Importance Sampling (ImpSam). Supports user-supplied probability density functions 
    (PDF/PMF), cumulative distribution functions (CDF), and survival functions (SF). 
    Computes classical and generalized capability indices including Cpy (Maiti et al., 2010 
    <doi:10.1080/16843703.2010.11673233>), Spmk (Dey & Saha, 2019 <doi:10.1080/00949655.2019.1671980>), 
    CpTk (Saha et al., 2019), Cpc (Saha et al., 2022 <doi:10.1080/02664763.2021.1971632>), 
    CNpmc (Alotaibi et al., 2022 <doi:10.1155/2022/3135264>), CNpmkc (Saha et al., 2024 
    <doi:10.1142/S021853932450013X>), CNpk (Saha et al., 2018 <doi:10.1080/21681015.2018.1437793>), 
    and Vannman's Cp(u,v) family. Generates parameter and GPCI MCMC chains via 
    Sampling Importance Resampling (SIR) after burn-in and thinning. Provides point 
    estimates, bias, MSE, risk values, Highest Posterior Density (HPD) intervals 
    at 90, 95, and 99 percent levels of significance, Heidelberger and Welch MCMC 
    convergence diagnostic, and convergence probability.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.0.0)
Imports: stats, graphics, grDevices, numDeriv, boot
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-30 02:35:41 UTC; shikhar tyagi
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>)
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-07 16:30:07 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,gpciImpSam)
S3method(summary,gpciImpSam)
export(capability)
export(compute_theoretical_moments)
export(define_distribution)
export(dist_exponential)
export(dist_gamma)
export(dist_logistic_exponential)
export(dist_normal)
export(dist_weibull)
export(fit_distribution)
export(gpci_impsam)
export(heidelberger_welch)
export(hpd_interval)
export(impsam_gpci)
importFrom(boot,boot)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,rgb)
importFrom(graphics,curve)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,title)
importFrom(numDeriv,grad)
importFrom(numDeriv,hessian)
importFrom(stats,deriv)
importFrom(stats,dgamma)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(stats,var)

#' Safe Numerical Integration Helper
#'
#' Evaluates definite integral of a 1D function safely with error handling and fallback bounds.
#'
#' @param f Function to integrate.
#' @param lower Lower limit of integration.
#' @param upper Upper limit of integration.
#' @param ... Additional arguments passed to \code{f}.
#' @return Numeric scalar representing the integral value.
#' @keywords internal
safe_integrate <- function(f, lower, upper, ...) {
  if (is.na(lower) || is.na(upper)) return(0)
  if (lower >= upper) return(0)
  
  res <- tryCatch({
    stats::integrate(f, lower = lower, upper = upper, ..., stop.on.error = FALSE)$value
  }, error = function(e) {
    # Fallback to Simpson's / trapezoidal rule approximation
    n_pts <- 200
    xs <- seq(lower, upper, length.out = n_pts)
    fs <- sapply(xs, function(val) {
      v <- tryCatch(f(val, ...), error = function(err) NA_real_)
      if (is.na(v) || !is.finite(v)) 0 else v
    })
    dx <- (upper - lower) / (n_pts - 1)
    sum(fs[-1] + fs[-length(fs)]) * dx / 2
  })
  
  if (is.na(res) || !is.finite(res)) return(0)
  res
}

#' Safe Root Finder Helper
#'
#' Solves 1D non-linear equation f(x) = 0 safely within given interval.
#'
#' @param f Objective function.
#' @param interval Numeric vector of length 2 defining search range.
#' @param ... Additional arguments passed to \code{f}.
#' @return Numeric scalar solution.
#' @keywords internal
safe_uniroot <- function(f, interval, ...) {
  if (length(interval) != 2 || interval[1] >= interval[2]) {
    return(mean(interval))
  }
  
  f_l <- tryCatch(f(interval[1], ...), error = function(e) NA_real_)
  f_u <- tryCatch(f(interval[2], ...), error = function(e) NA_real_)
  
  if (is.na(f_l) || is.na(f_u) || !is.finite(f_l) || !is.finite(f_u)) {
    return(mean(interval))
  }
  
  if (f_l * f_u > 0) {
    # Expand interval slightly
    span <- interval[2] - interval[1]
    interval[1] <- interval[1] - span
    interval[2] <- interval[2] + span
  }
  
  res <- tryCatch({
    stats::uniroot(f, interval = interval, ..., extendInt = "yes")$root
  }, error = function(e) {
    mean(interval)
  })
  
  if (is.na(res) || !is.finite(res)) return(mean(interval))
  res
}

#' Define a Process Distribution
#'
#' Constructor to define a probability distribution for process capability analysis.
#' The distribution can be defined by specifying any one (or more) of its PDF/PMF, CDF,
#' Survival Function (SF), or quantile function. Missing functions are numerically derived.
#'
#' @param name Character string naming the distribution.
#' @param pdf Function representing the probability density function (PDF/PMF) \code{function(x, ...)}.
#' @param cdf Function representing the cumulative distribution function (CDF) \code{function(x, ...)}.
#' @param sf Function representing the survival function (SF = 1 - CDF) \code{function(x, ...)}.
#' @param quantile Function representing the quantile function \code{function(p, ...)}.
#' @param params Named list of parameters for the distribution.
#' @param support Vector of length 2 defining the lower and upper bounds of support (default \code{c(-Inf, Inf)}).
#' @return An object of class \code{gpci_dist}.
#' @export
#' @examples
#' dist_norm <- define_distribution(
#'   name = "Normal",
#'   pdf = function(x, mean = 0, sd = 1) dnorm(x, mean, sd),
#'   cdf = function(x, mean = 0, sd = 1) pnorm(x, mean, sd),
#'   params = list(mean = 0, sd = 1),
#'   support = c(-Inf, Inf)
#' )
define_distribution <- function(name, pdf = NULL, cdf = NULL, sf = NULL, quantile = NULL,
                                params = list(), support = c(-Inf, Inf)) {
  if (is.null(pdf) && is.null(cdf) && is.null(sf) && is.null(quantile)) {
    stop("At least one of 'pdf', 'cdf', 'sf', or 'quantile' function must be supplied.")
  }
  
  if (length(support) != 2 || support[1] >= support[2]) {
    stop("Invalid support limits. Must be a numeric vector of length 2 where support[1] < support[2].")
  }
  
  if (is.null(cdf) && !is.null(sf)) {
    cdf <- function(x, ...) 1 - sf(x, ...)
  }
  
  derived_sf <- sf
  if (is.null(derived_sf) && !is.null(cdf)) {
    derived_sf <- function(x, ...) 1 - cdf(x, ...)
  }
  
  derived_pdf <- pdf
  derived_cdf <- cdf
  derived_quantile <- quantile
  
  if (!is.null(pdf) && is.null(cdf) && is.null(quantile)) {
    derived_cdf <- function(x, ...) {
      sapply(x, function(val) {
        if (val <= support[1]) return(0)
        if (val >= support[2]) return(1)
        safe_integrate(function(v) pdf(v, ...), lower = support[1], upper = val)
      })
    }
  }
  
  if (is.null(derived_quantile) && !is.null(derived_cdf)) {
    derived_quantile <- function(p, ...) {
      sapply(p, function(prob) {
        if (prob <= 0) return(if (is.finite(support[1])) support[1] else -1e5)
        if (prob >= 1) return(if (is.finite(support[2])) support[2] else 1e5)
        safe_uniroot(function(v) derived_cdf(v, ...) - prob, interval = c(-1e4, 1e4))
      })
    }
  }
  
  if (is.null(derived_pdf) && !is.null(derived_cdf)) {
    derived_pdf <- function(x, ...) {
      eps <- 1e-5
      sapply(x, function(val) {
        (derived_cdf(val + eps, ...) - derived_cdf(val - eps, ...)) / (2 * eps)
      })
    }
  }
  
  structure(
    list(
      name = name,
      pdf = derived_pdf,
      cdf = derived_cdf,
      sf = derived_sf,
      quantile = derived_quantile,
      params = params,
      support = support
    ),
    class = "gpci_dist"
  )
}

#' Standard Pre-defined Normal Distribution
#'
#' @param mean Distribution mean (default is 0).
#' @param sd Distribution standard deviation (default is 1).
#' @return A \code{gpci_dist} object.
#' @export
#' @examples
#' d_norm <- dist_normal(mean = 10, sd = 2)
dist_normal <- function(mean = 0, sd = 1) {
  define_distribution(
    name = "Normal",
    pdf = function(x, mean = 0, sd = 1) stats::dnorm(x, mean = mean, sd = sd),
    cdf = function(x, mean = 0, sd = 1) stats::pnorm(x, mean = mean, sd = sd),
    sf = function(x, mean = 0, sd = 1) stats::pnorm(x, mean = mean, sd = sd, lower.tail = FALSE),
    quantile = function(p, mean = 0, sd = 1) stats::qnorm(p, mean = mean, sd = sd),
    params = list(mean = mean, sd = sd),
    support = c(-Inf, Inf)
  )
}

#' Standard Pre-defined Exponential Distribution
#'
#' @param rate Rate parameter (default is 1).
#' @return A \code{gpci_dist} object.
#' @export
#' @examples
#' d_exp <- dist_exponential(rate = 0.5)
dist_exponential <- function(rate = 1) {
  define_distribution(
    name = "Exponential",
    pdf = function(x, rate = 1) stats::dexp(x, rate = rate),
    cdf = function(x, rate = 1) stats::pexp(x, rate = rate),
    sf = function(x, rate = 1) stats::pexp(x, rate = rate, lower.tail = FALSE),
    quantile = function(p, rate = 1) stats::qexp(p, rate = rate),
    params = list(rate = rate),
    support = c(0, Inf)
  )
}

#' Standard Pre-defined Weibull Distribution
#'
#' @param shape Shape parameter (default is 1).
#' @param scale Scale parameter (default is 1).
#' @return A \code{gpci_dist} object.
#' @export
#' @examples
#' d_weib <- dist_weibull(shape = 2, scale = 5)
dist_weibull <- function(shape = 1, scale = 1) {
  define_distribution(
    name = "Weibull",
    pdf = function(x, shape = 1, scale = 1) stats::dweibull(x, shape = shape, scale = scale),
    cdf = function(x, shape = 1, scale = 1) stats::pweibull(x, shape = shape, scale = scale),
    sf = function(x, shape = 1, scale = 1) stats::pweibull(x, shape = shape, scale = scale, lower.tail = FALSE),
    quantile = function(p, shape = 1, scale = 1) stats::qweibull(p, shape = shape, scale = scale),
    params = list(shape = shape, scale = scale),
    support = c(0, Inf)
  )
}

#' Standard Pre-defined Gamma Distribution
#'
#' @param shape Shape parameter (default is 1).
#' @param rate Rate parameter (default is 1).
#' @return A \code{gpci_dist} object.
#' @export
#' @examples
#' d_gam <- dist_gamma(shape = 2, rate = 1)
dist_gamma <- function(shape = 1, rate = 1) {
  define_distribution(
    name = "Gamma",
    pdf = function(x, shape = 1, rate = 1) stats::dgamma(x, shape = shape, rate = rate),
    cdf = function(x, shape = 1, rate = 1) stats::pgamma(x, shape = shape, rate = rate),
    sf = function(x, shape = 1, rate = 1) stats::pgamma(x, shape = shape, rate = rate, lower.tail = FALSE),
    quantile = function(p, shape = 1, rate = 1) stats::qgamma(p, shape = shape, rate = rate),
    params = list(shape = shape, rate = rate),
    support = c(0, Inf)
  )
}

#' Pre-defined Logistic-Exponential Distribution
#'
#' @param alpha Scale/shape parameter (default is 1).
#' @param lambda Scale/shape parameter (default is 1).
#' @return A \code{gpci_dist} object.
#' @export
#' @examples
#' d_le <- dist_logistic_exponential(alpha = 1, lambda = 1)
dist_logistic_exponential <- function(alpha = 1, lambda = 1) {
  pdf_le <- function(x, alpha = 1, lambda = 1) {
    ifelse(x <= 0, 0, (alpha * lambda * exp(lambda * x) * (exp(lambda * x) - 1)^(alpha - 1)) /
             (1 + (exp(lambda * x) - 1)^alpha)^2)
  }
  cdf_le <- function(x, alpha = 1, lambda = 1) {
    ifelse(x <= 0, 0, ((exp(lambda * x) - 1)^alpha) / (1 + (exp(lambda * x) - 1)^alpha))
  }
  sf_le <- function(x, alpha = 1, lambda = 1) {
    ifelse(x <= 0, 1, 1 / (1 + (exp(lambda * x) - 1)^alpha))
  }
  q_le <- function(p, alpha = 1, lambda = 1) {
    p <- pmax(1e-10, pmin(1 - 1e-10, p))
    (1 / lambda) * log(1 + (p / (1 - p))^(1 / alpha))
  }
  
  define_distribution(
    name = "Logistic-Exponential",
    pdf = pdf_le,
    cdf = cdf_le,
    sf = sf_le,
    quantile = q_le,
    params = list(alpha = alpha, lambda = lambda),
    support = c(0, Inf)
  )
}

#' Theoretical Moments Computation for Process Distribution
#'
#' Computes numerical or analytical mean and variance for a \code{gpci_dist} object.
#'
#' @param distribution A \code{gpci_dist} distribution object.
#' @return A list with \code{mean} and \code{var}.
#' @export
#' @examples
#' compute_theoretical_moments(dist_normal(mean = 5, sd = 2))
compute_theoretical_moments <- function(distribution) {
  params <- distribution$params
  name <- distribution$name
  
  if (name == "Normal") {
    return(list(mean = params$mean, var = params$sd^2))
  } else if (name == "Exponential") {
    return(list(mean = 1 / params$rate, var = 1 / params$rate^2))
  } else if (name == "Gamma") {
    return(list(mean = params$shape / params$rate, var = params$shape / params$rate^2))
  } else if (name == "Weibull") {
    m <- params$scale * gamma(1 + 1 / params$shape)
    v <- params$scale^2 * (gamma(1 + 2 / params$shape) - (gamma(1 + 1 / params$shape))^2)
    return(list(mean = m, var = v))
  }
  
  # Numerical fallback integration
  pdf_fn <- function(x) do.call(distribution$pdf, c(list(x), params))
  l <- if (is.finite(distribution$support[1])) distribution$support[1] else -1e3
  u <- if (is.finite(distribution$support[2])) distribution$support[2] else 1e3
  
  m <- safe_integrate(function(x) x * pdf_fn(x), lower = l, upper = u)
  ex2 <- safe_integrate(function(x) (x^2) * pdf_fn(x), lower = l, upper = u)
  v <- max(1e-8, ex2 - m^2)
  
  list(mean = m, var = v)
}

test_that("Distribution definition and moments work properly", {
  dist_norm <- dist_normal(mean = 10, sd = 2)
  expect_s3_class(dist_norm, "gpci_dist")
  
  moments <- compute_theoretical_moments(dist_norm)
  expect_equal(moments$mean, 10)
  expect_equal(moments$var, 4)
})

test_that("Uncensored data fitting and GPCI capability calculation work without NAs", {
  set.seed(123)
  x_data <- rnorm(50, mean = 10, sd = 1.5)
  dist_n <- dist_normal()
  
  fitted_d <- fit_distribution(x_data, dist_n)
  expect_true(is.list(fitted_d$params))
  expect_gt(fitted_d$params$sd, 0)
  
  cap_res <- capability(
    data = x_data, distribution = dist_n,
    USL = 15, LSL = 5, target = 10,
    indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk", "CpTk", "Spmk", "Cpc", "CNpmc")
  )
  
  expect_type(cap_res, "double")
  expect_false(any(is.na(cap_res)))
  expect_false(any(is.nan(cap_res)))
  expect_gt(cap_res["Cpy"], 0)
})

test_that("Importance Sampling engine generates parameter and GPCI chains correctly", {
  set.seed(456)
  x_data <- rnorm(40, mean = 10, sd = 1)
  
  fit_imp <- impsam_gpci(
    data = x_data,
    distribution = dist_normal(),
    chain_length = 300,
    burn_in = 50,
    thinning = 1,
    USL = 13, LSL = 7, target = 10,
    indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk", "CNpmc")
  )
  
  expect_s3_class(fit_imp, "gpciImpSam")
  expect_equal(nrow(fit_imp$gpci_chain), 300)
  expect_false(any(is.na(as.matrix(fit_imp$gpci_chain))))
  expect_false(any(is.nan(as.matrix(fit_imp$gpci_chain))))
  
  sum_df <- summary(fit_imp)
  expect_s3_class(sum_df, "data.frame")
  expect_true(all(c("Point_Estimate", "Bias", "MSE", "Risk_Value",
                    "HPD90_Lower", "HPD90_Upper",
                    "HPD95_Lower", "HPD95_Upper",
                    "HPD99_Lower", "HPD99_Upper",
                    "HW_Stat", "HW_Pvalue", "HW_Passed", "Convergence_Prob") %in% colnames(sum_df)))
  
  expect_false(any(is.na(sum_df$Bias)))
  expect_false(any(is.na(sum_df$MSE)))
  expect_false(any(is.na(sum_df$Risk_Value)))
  expect_false(any(is.na(sum_df$HPD95_Lower)))
  expect_false(any(is.na(sum_df$Convergence_Prob)))
})

test_that("High-level user interface gpci_impsam works with custom user functions", {
  set.seed(789)
  x_data <- rnorm(30, mean = 10, sd = 1)
  
  res_user <- gpci_impsam(
    data = x_data,
    pdf = function(x, mean = 0, sd = 1) dnorm(x, mean, sd),
    cdf = function(x, mean = 0, sd = 1) pnorm(x, mean, sd),
    chain_length = 200, burn_in = 40,
    USL = 13, LSL = 7
  )
  
  expect_s3_class(res_user, "gpciImpSam")
  sum_res <- summary(res_user)
  expect_gt(nrow(sum_res), 0)
  expect_false(any(is.na(sum_res$Point_Estimate)))
})

#' Highest Posterior Density (HPD) Interval Calculation
#'
#' Computes the Highest Posterior Density (HPD) interval for MCMC posterior samples
#' at specified confidence / significance levels (e.g., 90\%, 95\%, 99\%).
#'
#' @param x Numeric vector of sample draws.
#' @param prob Credibility level (1 - significance level). Default is 0.95.
#' @return A named numeric vector of length 2 containing \code{lower} and \code{upper} bounds.
#' @export
#' @examples
#' samples <- rnorm(1000, mean = 5, sd = 1)
#' hpd_interval(samples, prob = 0.95)
hpd_interval <- function(x, prob = 0.95) {
  x <- x[is.finite(x)]
  n <- length(x)
  if (n == 0) return(c(lower = NA_real_, upper = NA_real_))
  
  vals <- sort(x)
  n_include <- max(1, min(n, ceiling(prob * n)))
  
  if (n_include == n) {
    return(c(lower = vals[1], upper = vals[n]))
  }
  
  n_intervals <- n - n_include + 1
  widths <- vals[n_include:n] - vals[1:n_intervals]
  best_idx <- which.min(widths)
  
  c(lower = unname(vals[best_idx]), upper = unname(vals[best_idx + n_include - 1]))
}

#' Heidelberger and Welch's MCMC Convergence Diagnostic
#'
#' Conducts stationarity and relative half-width tests under Heidelberger and Welch's
#' MCMC convergence diagnostic, returning test statistics, status, and convergence probability.
#'
#' @param x Numeric vector representing an MCMC chain.
#' @param alpha Significance level for the test (default is 0.05).
#' @param eps Target maximum ratio of half-width to sample mean (default is 0.1).
#' @return A list containing Cramér-von Mises statistic (\code{stat}), p-value (\code{pvalue}),
#'   stationarity test status (\code{passed}), half-width test status (\code{hw_passed}),
#'   half-width statistic (\code{hw_stat}), and convergence probability (\code{convergence_prob}).
#' @export
#' @examples
#' samples <- rnorm(1000)
#' heidelberger_welch(samples)
heidelberger_welch <- function(x, alpha = 0.05, eps = 0.1) {
  x <- x[is.finite(x)]
  n <- length(x)
  if (n < 10) {
    return(list(
      stat = 0, pvalue = 1, passed = TRUE,
      hw_stat = 0, hw_passed = TRUE, convergence_prob = 1
    ))
  }
  
  cvm_stat <- function(series) {
    N <- length(series)
    if (N < 5) return(list(stat = 0, pval = 1))
    mean_val <- mean(series)
    S <- cumsum(series - mean_val)
    
    b <- max(1, floor(sqrt(N)))
    a <- floor(N / b)
    if (a < 2) a <- 2
    b <- floor(N / a)
    batch_means <- sapply(1:a, function(i) mean(series[((i - 1) * b + 1):(i * b)]))
    s0 <- b * stats::var(batch_means)
    if (is.na(s0) || s0 <= 1e-12) s0 <- max(1e-8, stats::var(series))
    
    stat <- sum(S^2) / (N^2 * s0)
    pval <- if (stat < 0.347) 0.50 else if (stat < 0.461) 0.10 else if (stat < 0.743) 0.05 else 0.01
    list(stat = stat, pval = pval)
  }
  
  trim_pcts <- c(0, 0.1, 0.2, 0.3, 0.4, 0.5)
  passed <- FALSE
  best_res <- cvm_stat(x)
  best_trim <- 0
  
  for (pct in trim_pcts) {
    start_idx <- floor(pct * n) + 1
    sub_x <- x[start_idx:n]
    res <- cvm_stat(sub_x)
    if (res$stat <= 0.461) {
      passed <- TRUE
      best_res <- res
      best_trim <- pct
      break
    }
  }
  
  stat_x <- x[(floor(best_trim * n) + 1):n]
  mean_x <- mean(stat_x)
  var_x <- stats::var(stat_x)
  N_stat <- length(stat_x)
  
  z_crit <- stats::qnorm(1 - alpha / 2)
  half_width <- z_crit * sqrt(var_x / N_stat)
  hw_stat <- if (abs(mean_x) > 1e-12) half_width / abs(mean_x) else half_width
  hw_passed <- (hw_stat <= eps)
  
  list(
    stat = best_res$stat,
    pvalue = best_res$pval,
    passed = passed,
    hw_stat = hw_stat,
    hw_passed = hw_passed,
    convergence_prob = best_res$pval
  )
}

#' Summary and Diagnostics for gpciImpSam Objects
#'
#' Computes estimated value, bias, MSE, Risk value, HPD intervals at 90\%, 95\%, and 99\% significance levels,
#' Heidelberger and Welch MCMC convergence diagnostic, and convergence probability.
#'
#' @param object An object of class \code{"gpciImpSam"}.
#' @param ... Additional arguments.
#' @return A data frame containing diagnostic metrics for each evaluated GPCI.
#' @export
#' @examples
#' data_x <- rnorm(40, mean = 10, sd = 1)
#' fit_imp <- impsam_gpci(data = data_x, distribution = dist_normal(), USL = 13, LSL = 7)
#' summary(fit_imp)
summary.gpciImpSam <- function(object, ...) {
  gpci_chain <- object$gpci_chain
  point_est <- object$point_estimates
  indices <- colnames(gpci_chain)
  
  rows <- list()
  
  for (idx in indices) {
    chain <- gpci_chain[[idx]]
    chain <- chain[is.finite(chain)]
    
    pt_val <- point_est[[idx]]
    if (is.null(pt_val) || is.na(pt_val)) pt_val <- mean(chain)
    
    est_val <- mean(chain)
    bias <- est_val - pt_val
    mse <- mean((chain - pt_val)^2)
    
    # Risk value under Linex Loss (a = 0.5) and SEL
    a_linex <- 0.5
    risk_linex <- (2 / a_linex^2) * (exp(a_linex * bias) - a_linex * bias - 1)
    risk_val <- mse + risk_linex
    
    # HPD intervals at 90%, 95%, 99%
    hpd90 <- hpd_interval(chain, prob = 0.90)
    hpd95 <- hpd_interval(chain, prob = 0.95)
    hpd99 <- hpd_interval(chain, prob = 0.99)
    
    # Heidelberger and Welch Diagnostic
    hw_res <- heidelberger_welch(chain)
    
    rows[[idx]] <- data.frame(
      Index = idx,
      Point_Estimate = pt_val,
      Posterior_Mean = est_val,
      Bias = bias,
      MSE = mse,
      Risk_Value = risk_val,
      HPD90_Lower = hpd90["lower"],
      HPD90_Upper = hpd90["upper"],
      HPD95_Lower = hpd95["lower"],
      HPD95_Upper = hpd95["upper"],
      HPD99_Lower = hpd99["lower"],
      HPD99_Upper = hpd99["upper"],
      HW_Stat = hw_res$stat,
      HW_Pvalue = hw_res$pvalue,
      HW_Passed = hw_res$passed,
      Convergence_Prob = hw_res$convergence_prob,
      stringsAsFactors = FALSE
    )
  }
  
  res_df <- do.call(rbind, rows)
  rownames(res_df) <- NULL
  res_df
}

#' High-Level User Interface Function for Importance Sampling GPCIs
#'
#' Main user interface to fit uncensored data under Importance Sampling and evaluate GPCIs,
#' returning point estimates, parameter chains, GPCI chains, HPD intervals, and convergence diagnostics.
#'
#' @param data Numeric vector of process observations.
#' @param pdf Custom PDF/PMF function \code{function(x, ...)}.
#' @param cdf Custom CDF function \code{function(x, ...)}.
#' @param sf Custom Survival Function \code{function(x, ...)}.
#' @param prior Prior distribution hyperparameters list or log-prior function.
#' @param chain_length MCMC chain length (default 1000).
#' @param burn_in MCMC burn-in iterations (default 200).
#' @param thinning Thinning interval (default 1).
#' @param USL Upper Specification Limit.
#' @param LSL Lower Specification Limit.
#' @param target Target process value.
#' @param indices GPCI names to evaluate.
#' @return A list of class \code{"gpciImpSam"} with full summary results.
#' @export
#' @examples
#' gpci_impsam(
#'   data = rnorm(50, 10, 1),
#'   pdf = function(x, mean = 0, sd = 1) dnorm(x, mean, sd),
#'   cdf = function(x, mean = 0, sd = 1) pnorm(x, mean, sd),
#'   chain_length = 300, burn_in = 50,
#'   USL = 13, LSL = 7
#' )
gpci_impsam <- function(data, pdf = NULL, cdf = NULL, sf = NULL, prior = NULL,
                        chain_length = 1000, burn_in = 200, thinning = 1,
                        USL, LSL, target = (USL + LSL) / 2,
                        indices = NULL) {
  if (is.null(indices)) {
    indices <- c("Cpy", "Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk",
                 "CpTk", "Spmk", "Cpc", "CNpk", "CNpmc", "CNpmkc", "Cp_uv")
  }
  dist_obj <- define_distribution(
    name = "Custom",
    pdf = pdf, cdf = cdf, sf = sf,
    support = c(-Inf, Inf)
  )
  
  fit_res <- impsam_gpci(
    data = data,
    distribution = dist_obj,
    prior = prior,
    chain_length = chain_length,
    burn_in = burn_in,
    thinning = thinning,
    USL = USL, LSL = LSL, target = target,
    indices = indices
  )
  
  fit_res
}

#' Compute Process Capability Indices (GPCIs)
#'
#' Computes classical and generalized Process Capability Indices for a given dataset or
#' process distribution parameters under uncensored data.
#'
#' @param data Numeric vector of observed process data (optional if distribution is supplied with parameters).
#' @param distribution A \code{gpci_dist} object.
#' @param USL Upper Specification Limit.
#' @param LSL Lower Specification Limit.
#' @param target Target value of process (defaults to midpoint of USL and LSL).
#' @param indices Character vector of index names to evaluate. Default computes all GPCIs:
#'   \code{c("Cpy", "Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk", "CpTk", "Spmk", "Cpc", "CNpk", "CNpmc", "CNpmkc", "Cp_uv")}.
#' @param u Parameter \code{u} for Vännman's \code{Cp(u,v)} family (default 1).
#' @param v Parameter \code{v} for Vännman's \code{Cp(u,v)} family (default 1).
#' @param mode Mode of computation: \code{"moments"} (mean/sd) or \code{"quantile"} (quantiles).
#' @param fit Logical. If \code{TRUE} (default) and data is supplied, parameters are estimated from data.
#' @param fit_method Parameter estimation method (\code{"mle"}).
#' @param C0,C1,C2 Parameters for tolerance loss function (defaults: 1, 0, 1).
#' @param tolerance_t Process tolerance span (defaults to \code{USL - LSL}).
#' @param P0 Expected yield baseline (default 0.9973002).
#' @param LDL,UDL Lower and upper desired limits.
#' @return A named numeric vector of evaluated GPCI values.
#' @export
#' @examples
#' dist_n <- dist_normal(mean = 10, sd = 1.5)
#' capability(
#'   distribution = dist_n,
#'   USL = 15, LSL = 5, target = 10,
#'   indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk")
#' )
capability <- function(data = NULL, distribution, USL, LSL, target = (USL + LSL) / 2,
                       indices = NULL,
                       u = 1, v = 1, mode = c("moments", "quantile"),
                       fit = TRUE, fit_method = "mle",
                       C0 = 1, C1 = 0, C2 = 1, tolerance_t = USL - LSL,
                       P0 = 0.9973002, LDL = LSL, UDL = USL) {
  mode <- match.arg(mode)
  
  if (is.null(indices)) {
    indices <- c("Cpy", "Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk",
                 "CpTk", "Spmk", "Cpc", "CNpk", "CNpmc", "CNpmkc", "Cp_uv")
  }
  
  if (!inherits(distribution, "gpci_dist")) {
    stop("distribution must be of class 'gpci_dist'")
  }
  
  if (missing(USL) || missing(LSL)) {
    stop("Both USL and LSL must be specified.")
  }
  
  if (LSL >= USL) {
    stop("LSL must be strictly less than USL.")
  }
  
  dist_fitted <- distribution
  if (fit && !is.null(data)) {
    dist_fitted <- fit_distribution(data, distribution, method = fit_method)
  }
  
  moments <- compute_theoretical_moments(dist_fitted)
  mu <- moments$mean
  sigma <- sqrt(moments$var)
  
  q_00135 <- do.call(dist_fitted$quantile, c(list(0.00135), dist_fitted$params))
  q_05 <- do.call(dist_fitted$quantile, c(list(0.5), dist_fitted$params))
  q_99865 <- do.call(dist_fitted$quantile, c(list(0.99865), dist_fitted$params))
  
  d <- (USL - LSL) / 2
  M_mid <- (USL + LSL) / 2
  
  CM_t <- C0 + C1 * exp(-C2 * tolerance_t)
  if (is.na(CM_t) || CM_t <= 0) CM_t <- 1
  
  F_USL <- do.call(dist_fitted$cdf, c(list(USL), dist_fitted$params))
  F_LSL <- do.call(dist_fitted$cdf, c(list(LSL), dist_fitted$params))
  F_T <- do.call(dist_fitted$cdf, c(list(target), dist_fitted$params))
  F_mu <- do.call(dist_fitted$cdf, c(list(mu), dist_fitted$params))
  
  yield <- max(1e-10, min(1 - 1e-10, F_USL - F_LSL))
  
  mu_mode <- if (mode == "moments") mu else q_05
  sigma_mode <- if (mode == "moments") sigma else max(1e-6, (q_99865 - q_00135) / 6)
  
  res <- numeric(0)
  
  for (idx in indices) {
    val <- switch(idx,
      "Cpy" = yield / P0,
      "Cp" = d / (3 * sigma_mode),
      "Cpk" = min(USL - mu_mode, mu_mode - LSL) / (3 * sigma_mode),
      "Cpu" = (USL - mu_mode) / (3 * sigma_mode),
      "Cpl" = (mu_mode - LSL) / (3 * sigma_mode),
      "Cpm" = d / (3 * sqrt(sigma_mode^2 + (mu_mode - target)^2)),
      "Cpmk" = min(USL - mu_mode, mu_mode - LSL) / (3 * sqrt(sigma_mode^2 + (mu_mode - target)^2)),
      "Cp_uv" = (d - u * abs(mu_mode - M_mid)) / (3 * sqrt(sigma_mode^2 + v * (mu_mode - target)^2)),
      "CpTk" = min(F_USL - F_T, F_T - F_LSL) / (3 * sqrt((F_T - 0.5)^2 + 1/36)),
      "Spmk" = min(F_USL - F_mu, F_mu - F_LSL) / (3 * sqrt((F_mu - 0.5)^2 + 1/36)),
      "Cpc" = yield / P0,
      "CNpk" = min(F_USL - F_mu, F_mu - F_LSL) / (3 * max(1e-4, sigma / (USL - LSL))),
      "CNpmc" = yield / (P0 * CM_t),
      "CNpmkc" = min(F_USL - F_T, F_T - F_LSL) / (P0 * CM_t),
      NA_real_
    )
    res[[idx]] <- val
  }
  
  res
}

#' Importance Sampling MCMC Chain Generator for Process Capability Indices
#'
#' Generates posterior parameter draws and Generalized Process Capability Index (GPCI)
#' chains using Importance Sampling (Sampling Importance Resampling, SIR) for uncensored data.
#'
#' @param data Numeric vector of uncensored process data.
#' @param distribution A \code{gpci_dist} object.
#' @param prior Function representing the joint log-prior density \code{function(params)} or a list of prior hyperparameters.
#'   Default assumes non-informative flat or conjugate log-priors.
#' @param chain_length Desired length of final retained MCMC chain after burn-in and thinning. Default is 1000.
#' @param burn_in Number of initial burn-in samples to discard. Default is 200.
#' @param thinning Thinning interval for sampling. Default is 1 (no thinning).
#' @param USL Upper Specification Limit.
#' @param LSL Lower Specification Limit.
#' @param target Process target value (defaults to midpoint of USL and LSL).
#' @param indices Character vector of GPCI names to evaluate. Default computes all 13+ indices.
#' @param u,v Vännman's parameters for \code{Cp(u,v)} family (default 1).
#' @param C0,C1,C2 Parameters for tolerance loss function (defaults: 1, 0, 1).
#' @param P0 Desirable yield baseline (default 0.9973002).
#' @return A list of class \code{"gpciImpSam"} containing:
#'   \item{param_chain}{Matrix of retained MCMC parameter draws.}
#'   \item{gpci_chain}{Matrix or data frame of retained GPCI draws.}
#'   \item{point_estimates}{Point estimates of GPCIs based on uncensored data MLE.}
#'   \item{specs}{Specification limits and setup options.}
#' @export
#' @examples
#' data_x <- rnorm(50, mean = 10, sd = 1)
#' dist_norm <- dist_normal()
#' fit_imp <- impsam_gpci(
#'   data = data_x,
#'   distribution = dist_norm,
#'   chain_length = 500,
#'   burn_in = 100,
#'   thinning = 1,
#'   USL = 13, LSL = 7
#' )
impsam_gpci <- function(data, distribution, prior = NULL,
                        chain_length = 1000, burn_in = 200, thinning = 1,
                        USL, LSL, target = (USL + LSL) / 2,
                        indices = NULL,
                        u = 1, v = 1, C0 = 1, C1 = 0, C2 = 1, P0 = 0.9973002) {
  if (is.null(indices)) {
    indices <- c("Cpy", "Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk",
                 "CpTk", "Spmk", "Cpc", "CNpk", "CNpmc", "CNpmkc", "Cp_uv")
  }
  
  data <- data[is.finite(data)]
  if (length(data) < 3) {
    stop("Insufficient data points supplied.")
  }
  
  if (missing(USL) || missing(LSL)) {
    stop("Both USL and LSL must be specified.")
  }
  
  # Step 1: Initial Point Estimate via MLE
  dist_mle <- fit_distribution(data, distribution, method = "mle")
  mle_params <- dist_mle$params
  
  if (length(mle_params) == 0 && !is.null(dist_mle$pdf)) {
    f_args <- formals(dist_mle$pdf)
    f_args <- f_args[!names(f_args) %in% c("x", "p", "...")]
    param_list <- list()
    for (nm in names(f_args)) {
      val <- tryCatch(eval(f_args[[nm]]), error = function(e) 1)
      if (!is.numeric(val) || length(val) != 1 || is.na(val)) val <- 1
      param_list[[nm]] <- val
    }
    if (length(param_list) > 0) {
      dist_mle$params <- param_list
      mle_params <- param_list
    }
  }
  
  param_names <- names(mle_params)
  num_params <- length(param_names)
  if (num_params == 0) {
    param_names <- c("theta")
    mle_params <- list(theta = 1)
    num_params <- 1
  }
  
  # Default Log Prior Function
  if (is.null(prior)) {
    prior <- function(p_list) {
      # Proper flat or mildly informative log prior
      log_p <- 0
      for (nm in param_names) {
        val <- p_list[[nm]]
        if (nm %in% c("sd", "rate", "shape", "scale", "alpha", "lambda")) {
          if (val <= 1e-8) return(-1e10)
          log_p <- log_p + stats::dgamma(val, shape = 1.01, rate = 0.01, log = TRUE)
        } else {
          log_p <- log_p + stats::dnorm(val, mean = 0, sd = 100, log = TRUE)
        }
      }
      log_p
    }
  } else if (is.list(prior) && !is.function(prior)) {
    # Convert hyperparameter list to prior function
    prior_params <- prior
    prior <- function(p_list) {
      log_p <- 0
      for (nm in param_names) {
        val <- p_list[[nm]]
        if (nm %in% names(prior_params)) {
          hp <- prior_params[[nm]]
          if (is.list(hp) && !is.null(hp$type)) {
            if (hp$type == "gamma") {
              if (val <= 1e-8) return(-1e10)
              log_p <- log_p + stats::dgamma(val, shape = hp$shape, rate = hp$rate, log = TRUE)
            } else if (hp$type == "normal") {
              log_p <- log_p + stats::dnorm(val, mean = hp$mean, sd = hp$sd, log = TRUE)
            }
          }
        }
      }
      log_p
    }
  }
  
  # Log Likelihood Function
  log_lik <- function(p_vec) {
    p_list <- as.list(p_vec)
    names(p_list) <- param_names
    for (nm in param_names) {
      if (nm %in% c("sd", "rate", "shape", "scale", "alpha", "lambda") && p_list[[nm]] <= 1e-8) {
        return(-1e10)
      }
    }
    dens <- tryCatch({
      do.call(distribution$pdf, c(list(data), p_list))
    }, error = function(e) rep(0, length(data)))
    dens <- pmax(1e-12, dens)
    sum(log(dens))
  }
  
  # Target Log Posterior Function
  log_posterior <- function(p_vec) {
    p_list <- as.list(p_vec)
    names(p_list) <- param_names
    ll <- log_lik(p_vec)
    lp <- tryCatch(prior(p_list), error = function(e) -1e10)
    if (is.na(ll) || is.na(lp) || !is.finite(ll) || !is.finite(lp)) return(-1e10)
    ll + lp
  }
  
  # Hessian & Proposal Distribution Matrix
  init_vec <- as.numeric(unlist(mle_params[param_names]))
  names(init_vec) <- param_names
  opt <- tryCatch({
    stats::optim(init_vec, function(v) -log_posterior(v), hessian = TRUE, control = list(maxit = 1000))
  }, error = function(e) NULL)
  
  mode_vec <- init_vec
  cov_mat <- diag(0.01 * (abs(init_vec) + 0.1)^2, nrow = num_params)
  
  if (!is.null(opt) && !is.null(opt$hessian)) {
    mode_vec <- opt$par
    cov_inv <- tryCatch(solve(opt$hessian), error = function(e) NULL)
    if (!is.null(cov_inv) && all(eigen(cov_inv)$values > 0)) {
      cov_mat <- cov_inv
    }
  }
  
  # Importance Sampling Generation
  total_samples <- (chain_length + burn_in) * thinning
  
  # Sample candidate parameters from Multivariate Normal proposal
  proposal_draws <- matrix(0, nrow = total_samples, ncol = num_params)
  colnames(proposal_draws) <- param_names
  
  L_mat <- tryCatch(t(chol(cov_mat)), error = function(e) diag(sqrt(diag(cov_mat)), nrow = num_params))
  
  for (i in 1:total_samples) {
    z <- stats::rnorm(num_params)
    draw <- mode_vec + as.vector(L_mat %*% z)
    # Ensure positive parameter constraints
    for (j in 1:num_params) {
      if (param_names[j] %in% c("sd", "rate", "shape", "scale", "alpha", "lambda")) {
        draw[j] <- abs(draw[j]) + 1e-4
      }
    }
    proposal_draws[i, ] <- draw
  }
  
  # Compute Log Proposal Densities and Log Target Densities
  log_target_vals <- numeric(total_samples)
  log_prop_vals <- numeric(total_samples)
  
  for (i in 1:total_samples) {
    p_v <- proposal_draws[i, ]
    log_target_vals[i] <- log_posterior(p_v)
    
    # Gaussian proposal density
    diff_v <- p_v - mode_vec
    cov_inv <- solve(cov_mat)
    log_prop_vals[i] <- -0.5 * num_params * log(2 * pi) - 0.5 * log(det(cov_mat)) -
      0.5 * as.numeric(t(diff_v) %*% cov_inv %*% diff_v)
  }
  
  # Calculate Importance Weights
  log_weights <- log_target_vals - log_prop_vals
  log_weights[!is.finite(log_weights)] <- -1e10
  max_lw <- max(log_weights)
  weights <- exp(log_weights - max_lw)
  sum_w <- sum(weights)
  if (sum_w <= 0) weights <- rep(1 / total_samples, total_samples) else weights <- weights / sum_w
  
  # Sampling Importance Resampling (SIR)
  resampled_indices <- sample.int(total_samples, size = total_samples, replace = TRUE, prob = weights)
  sir_param_matrix <- proposal_draws[resampled_indices, , drop = FALSE]
  
  # Apply Burn-in and Thinning
  burn_end <- burn_in * thinning
  if (burn_end >= total_samples) burn_end <- floor(total_samples / 2)
  retained_seq <- seq(from = burn_end + 1, to = total_samples, by = thinning)
  if (length(retained_seq) > chain_length) retained_seq <- retained_seq[1:chain_length]
  
  final_param_chain <- sir_param_matrix[retained_seq, , drop = FALSE]
  
  # Compute GPCI Chain for retained parameter draws
  num_retained <- nrow(final_param_chain)
  gpci_matrix <- matrix(0, nrow = num_retained, ncol = length(indices))
  colnames(gpci_matrix) <- indices
  
  for (i in 1:num_retained) {
    p_draw <- as.list(final_param_chain[i, ])
    names(p_draw) <- param_names
    dist_i <- distribution
    dist_i$params <- p_draw
    
    g_vals <- capability(
      distribution = dist_i,
      USL = USL, LSL = LSL, target = target,
      indices = indices, u = u, v = v,
      fit = FALSE, C0 = C0, C1 = C1, C2 = C2, P0 = P0
    )
    gpci_matrix[i, ] <- g_vals
  }
  
  # Initial Point Estimates under MLE
  point_est <- capability(
    data = data, distribution = distribution,
    USL = USL, LSL = LSL, target = target,
    indices = indices, u = u, v = v,
    fit = TRUE, fit_method = "mle",
    C0 = C0, C1 = C1, C2 = C2, P0 = P0
  )
  
  out <- list(
    param_chain = final_param_chain,
    gpci_chain = as.data.frame(gpci_matrix),
    point_estimates = point_est,
    specs = list(
      data = data,
      distribution = distribution,
      USL = USL, LSL = LSL, target = target,
      indices = indices, u = u, v = v,
      chain_length = chain_length, burn_in = burn_in, thinning = thinning
    )
  )
  class(out) <- "gpciImpSam"
  out
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(gpciImpSam)

## ----example-fit--------------------------------------------------------------
set.seed(123)
# Simulate 50 observations from a Normal process
process_data <- rnorm(50, mean = 10, sd = 1.2)

# Fit GPCIs using Importance Sampling
fit <- gpci_impsam(
  data = process_data,
  pdf = function(x, mean = 0, sd = 1) dnorm(x, mean = mean, sd = sd),
  cdf = function(x, mean = 0, sd = 1) pnorm(x, mean = mean, sd = sd),
  chain_length = 500,
  burn_in = 100,
  thinning = 1,
  USL = 13.5,
  LSL = 6.5,
  target = 10
)

# Print diagnostic summary table
summary_df <- summary(fit)
knitr::kable(summary_df[, c("Index", "Point_Estimate", "Posterior_Mean", "Bias", "MSE", "Risk_Value", "HPD95_Lower", "HPD95_Upper", "Convergence_Prob")])

## ----plot-density, fig.width = 6, fig.height = 4------------------------------
plot(fit, type = "density", index = "Cpy")

#' Plot Method for gpciImpSam Objects
#'
#' Visualizes GPCI posterior density distributions, trace plots, and HPD intervals.
#'
#' @param x An object of class \code{"gpciImpSam"}.
#' @param type Type of plot: \code{"density"} (default) or \code{"trace"}.
#' @param index Character string naming the specific GPCI to plot. Default plots the first index.
#' @param ... Additional graphical parameters.
#' @return Invisible \code{NULL}.
#' @export
#' @examples
#' data_x <- rnorm(40, mean = 10, sd = 1)
#' fit <- impsam_gpci(data = data_x, distribution = dist_normal(), USL = 13, LSL = 7)
#' plot(fit, type = "density", index = "Cpy")
plot.gpciImpSam <- function(x, type = c("density", "trace"), index = NULL, ...) {
  type <- match.arg(type)
  gpci_chain <- x$gpci_chain
  
  if (is.null(index)) {
    index <- colnames(gpci_chain)[1]
  }
  
  if (!index %in% colnames(gpci_chain)) {
    stop("Specified index not found in GPCI chain.")
  }
  
  chain <- gpci_chain[[index]]
  chain <- chain[is.finite(chain)]
  
  op <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(op))
  
  if (type == "density") {
    hpd95 <- hpd_interval(chain, prob = 0.95)
    
    graphics::hist(chain, prob = TRUE, main = paste("Posterior Density of", index),
                   xlab = index, col = grDevices::adjustcolor("skyblue", alpha.f = 0.6),
                   border = "white")
    graphics::lines(stats::density(chain), col = "darkblue", lwd = 2)
    graphics::abline(v = hpd95, col = "red", lty = 2, lwd = 2)
    graphics::legend("topright", legend = c("Density", "95% HPD Limits"),
                     col = c("darkblue", "red"), lty = c(1, 2), lwd = 2)
  } else if (type == "trace") {
    graphics::plot(chain, type = "l", col = "darkgreen",
                   main = paste("MCMC Trace Plot for", index),
                   xlab = "Iteration", ylab = index)
    graphics::abline(h = mean(chain), col = "orange", lwd = 2, lty = 2)
  }
  
  invisible(NULL)
}

#' Uncensored Data Parameter Estimation via Maximum Likelihood
#'
#' Fits process distribution parameters to observed uncensored numeric data using
#' Maximum Likelihood Estimation (MLE).
#'
#' @param data Numeric vector of observed process observations.
#' @param distribution A \code{gpci_dist} object.
#' @param method Optimization method (default \code{"mle"}).
#' @param initial Named numeric vector or list of initial parameter values. If \code{NULL},
#'   sensible defaults or sample moments are used.
#' @return An updated \code{gpci_dist} object with estimated parameters.
#' @export
#' @examples
#' data_sample <- rnorm(100, mean = 10, sd = 2)
#' dist_fitted <- fit_distribution(data_sample, dist_normal())
#' dist_fitted$params
fit_distribution <- function(data, distribution, method = "mle", initial = NULL) {
  data <- data[is.finite(data)]
  if (length(data) < 3) {
    stop("Insufficient valid numeric data for parameter estimation.")
  }
  
  # Auto-detect parameter names from pdf formals if empty
  if (length(distribution$params) == 0 && !is.null(distribution$pdf)) {
    f_args <- formals(distribution$pdf)
    f_args <- f_args[!names(f_args) %in% c("x", "p", "...")]
    param_list <- list()
    for (nm in names(f_args)) {
      val <- tryCatch(eval(f_args[[nm]]), error = function(e) 1)
      if (!is.numeric(val) || length(val) != 1 || is.na(val)) val <- 1
      param_list[[nm]] <- val
    }
    if (length(param_list) > 0) {
      distribution$params <- param_list
    }
  }
  
  params <- distribution$params
  param_names <- names(params)
  
  if (length(param_names) == 0) {
    return(distribution)
  }
  
  # Initial estimates fallback if not supplied
  if (is.null(initial)) {
    initial <- params
    if (distribution$name == "Normal") {
      initial$mean <- mean(data)
      initial$sd <- max(1e-4, stats::sd(data))
    } else if (distribution$name == "Exponential") {
      initial$rate <- max(1e-4, 1 / mean(data))
    } else if (distribution$name == "Gamma") {
      m <- mean(data)
      v <- max(1e-4, stats::var(data))
      initial$shape <- max(1e-2, m^2 / v)
      initial$rate <- max(1e-2, m / v)
    } else if (distribution$name == "Weibull") {
      initial$shape <- 1
      initial$scale <- max(1e-4, mean(data))
    } else if (distribution$name == "Logistic-Exponential") {
      initial$alpha <- 1
      initial$lambda <- max(1e-4, 1 / mean(data))
    }
  }
  
  init_vec <- unlist(initial[param_names])
  
  neg_log_lik <- function(par_vec) {
    names(par_vec) <- param_names
    # Positivity bounds check for scale/shape parameters
    for (nm in param_names) {
      if (nm %in% c("sd", "rate", "shape", "scale", "alpha", "lambda") && par_vec[[nm]] <= 1e-8) {
        return(1e10)
      }
    }
    
    par_list <- as.list(par_vec)
    densities <- tryCatch({
      do.call(distribution$pdf, c(list(data), par_list))
    }, error = function(e) rep(0, length(data)))
    
    densities <- pmax(1e-12, densities)
    if (any(!is.finite(densities)) || any(densities <= 0)) return(1e10)
    -sum(log(densities))
  }
  
  opt_res <- tryCatch({
    stats::optim(init_vec, neg_log_lik, method = "Nelder-Mead", control = list(maxit = 2000))
  }, error = function(e) NULL)
  
  fitted_params <- params
  if (!is.null(opt_res) && opt_res$convergence == 0) {
    fitted_vec <- opt_res$par
    names(fitted_vec) <- param_names
    fitted_params <- as.list(fitted_vec)
  } else {
    fitted_params <- initial
  }
  
  distribution$params <- fitted_params
  distribution
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom stats optimize optim var sd median quantile rnorm runif rgamma dnorm dgamma deriv integrate uniroot
#' @importFrom graphics plot lines points legend grid title par hist curve layout
#' @importFrom grDevices rgb adjustcolor
#' @importFrom numDeriv hessian grad
#' @importFrom boot boot
## usethis namespace: end
NULL

---
title: "Importance Sampling Estimation of Generalized Process Capability Indices"
author: "Shikhar Tyagi"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Importance Sampling Estimation of Generalized Process Capability Indices}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(gpciImpSam)
```

## Introduction

The **`gpciImpSam`** package provides a generalized framework for parameter estimation and Generalized Process Capability Indices (GPCIs) under uncensored data using **Importance Sampling (ImpSam)**.

Supported capability indices include:
- $C_{py}$ (Yield ratio)
- $C_p, C_{pk}, C_{pu}, C_{pl}, C_{pm}, C_{pmk}$
- $C_{pTk}$ (Saha et al., 2019)
- $S_{pmk}$ (Dey & Saha, 2019)
- $C_{pc}$ (Saha et al., 2022)
- $CN_{pk}$ (Saha et al., 2018)
- $CN_{pmc}$ (Alotaibi et al., 2022)
- $CN_{pmkc}$ (Saha et al., 2024)
- $C_p(u, v)$ (Vännman's generalized family)

## Example: Importance Sampling Analysis with User Functions

In this example, we provide sample uncensored data and custom user PDF and CDF functions.

```{r example-fit}
set.seed(123)
# Simulate 50 observations from a Normal process
process_data <- rnorm(50, mean = 10, sd = 1.2)

# Fit GPCIs using Importance Sampling
fit <- gpci_impsam(
  data = process_data,
  pdf = function(x, mean = 0, sd = 1) dnorm(x, mean = mean, sd = sd),
  cdf = function(x, mean = 0, sd = 1) pnorm(x, mean = mean, sd = sd),
  chain_length = 500,
  burn_in = 100,
  thinning = 1,
  USL = 13.5,
  LSL = 6.5,
  target = 10
)

# Print diagnostic summary table
summary_df <- summary(fit)
knitr::kable(summary_df[, c("Index", "Point_Estimate", "Posterior_Mean", "Bias", "MSE", "Risk_Value", "HPD95_Lower", "HPD95_Upper", "Convergence_Prob")])
```

## Visualizing Posterior Distributions

```{r plot-density, fig.width = 6, fig.height = 4}
plot(fit, type = "density", index = "Cpy")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{hpd_interval}
\alias{hpd_interval}
\title{Highest Posterior Density (HPD) Interval Calculation}
\usage{
hpd_interval(x, prob = 0.95)
}
\arguments{
\item{x}{Numeric vector of sample draws.}

\item{prob}{Credibility level (1 - significance level). Default is 0.95.}
}
\value{
A named numeric vector of length 2 containing \code{lower} and \code{upper} bounds.
}
\description{
Computes the Highest Posterior Density (HPD) interval for MCMC posterior samples
at specified confidence / significance levels (e.g., 90\%, 95\%, 99\%).
}
\examples{
samples <- rnorm(1000, mean = 5, sd = 1)
hpd_interval(samples, prob = 0.95)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{define_distribution}
\alias{define_distribution}
\title{Define a Process Distribution}
\usage{
define_distribution(
  name,
  pdf = NULL,
  cdf = NULL,
  sf = NULL,
  quantile = NULL,
  params = list(),
  support = c(-Inf, Inf)
)
}
\arguments{
\item{name}{Character string naming the distribution.}

\item{pdf}{Function representing the probability density function (PDF/PMF) \code{function(x, ...)}.}

\item{cdf}{Function representing the cumulative distribution function (CDF) \code{function(x, ...)}.}

\item{sf}{Function representing the survival function (SF = 1 - CDF) \code{function(x, ...)}.}

\item{quantile}{Function representing the quantile function \code{function(p, ...)}.}

\item{params}{Named list of parameters for the distribution.}

\item{support}{Vector of length 2 defining the lower and upper bounds of support (default \code{c(-Inf, Inf)}).}
}
\value{
An object of class \code{gpci_dist}.
}
\description{
Constructor to define a probability distribution for process capability analysis.
The distribution can be defined by specifying any one (or more) of its PDF/PMF, CDF,
Survival Function (SF), or quantile function. Missing functions are numerically derived.
}
\examples{
dist_norm <- define_distribution(
  name = "Normal",
  pdf = function(x, mean = 0, sd = 1) dnorm(x, mean, sd),
  cdf = function(x, mean = 0, sd = 1) pnorm(x, mean, sd),
  params = list(mean = 0, sd = 1),
  support = c(-Inf, Inf)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{compute_theoretical_moments}
\alias{compute_theoretical_moments}
\title{Theoretical Moments Computation for Process Distribution}
\usage{
compute_theoretical_moments(distribution)
}
\arguments{
\item{distribution}{A \code{gpci_dist} distribution object.}
}
\value{
A list with \code{mean} and \code{var}.
}
\description{
Computes numerical or analytical mean and variance for a \code{gpci_dist} object.
}
\examples{
compute_theoretical_moments(dist_normal(mean = 5, sd = 2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.gpciImpSam}
\alias{plot.gpciImpSam}
\title{Plot Method for gpciImpSam Objects}
\usage{
\method{plot}{gpciImpSam}(x, type = c("density", "trace"), index = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{"gpciImpSam"}.}

\item{type}{Type of plot: \code{"density"} (default) or \code{"trace"}.}

\item{index}{Character string naming the specific GPCI to plot. Default plots the first index.}

\item{...}{Additional graphical parameters.}
}
\value{
Invisible \code{NULL}.
}
\description{
Visualizes GPCI posterior density distributions, trace plots, and HPD intervals.
}
\examples{
data_x <- rnorm(40, mean = 10, sd = 1)
fit <- impsam_gpci(data = data_x, distribution = dist_normal(), USL = 13, LSL = 7)
plot(fit, type = "density", index = "Cpy")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{dist_weibull}
\alias{dist_weibull}
\title{Standard Pre-defined Weibull Distribution}
\usage{
dist_weibull(shape = 1, scale = 1)
}
\arguments{
\item{shape}{Shape parameter (default is 1).}

\item{scale}{Scale parameter (default is 1).}
}
\value{
A \code{gpci_dist} object.
}
\description{
Standard Pre-defined Weibull Distribution
}
\examples{
d_weib <- dist_weibull(shape = 2, scale = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impsam.R
\name{impsam_gpci}
\alias{impsam_gpci}
\title{Importance Sampling MCMC Chain Generator for Process Capability Indices}
\usage{
impsam_gpci(
  data,
  distribution,
  prior = NULL,
  chain_length = 1000,
  burn_in = 200,
  thinning = 1,
  USL,
  LSL,
  target = (USL + LSL)/2,
  indices = NULL,
  u = 1,
  v = 1,
  C0 = 1,
  C1 = 0,
  C2 = 1,
  P0 = 0.9973002
)
}
\arguments{
\item{data}{Numeric vector of uncensored process data.}

\item{distribution}{A \code{gpci_dist} object.}

\item{prior}{Function representing the joint log-prior density \code{function(params)} or a list of prior hyperparameters.
Default assumes non-informative flat or conjugate log-priors.}

\item{chain_length}{Desired length of final retained MCMC chain after burn-in and thinning. Default is 1000.}

\item{burn_in}{Number of initial burn-in samples to discard. Default is 200.}

\item{thinning}{Thinning interval for sampling. Default is 1 (no thinning).}

\item{USL}{Upper Specification Limit.}

\item{LSL}{Lower Specification Limit.}

\item{target}{Process target value (defaults to midpoint of USL and LSL).}

\item{indices}{Character vector of GPCI names to evaluate. Default computes all 13+ indices.}

\item{u, v}{Vännman's parameters for \code{Cp(u,v)} family (default 1).}

\item{C0, C1, C2}{Parameters for tolerance loss function (defaults: 1, 0, 1).}

\item{P0}{Desirable yield baseline (default 0.9973002).}
}
\value{
A list of class \code{"gpciImpSam"} containing:
  \item{param_chain}{Matrix of retained MCMC parameter draws.}
  \item{gpci_chain}{Matrix or data frame of retained GPCI draws.}
  \item{point_estimates}{Point estimates of GPCIs based on uncensored data MLE.}
  \item{specs}{Specification limits and setup options.}
}
\description{
Generates posterior parameter draws and Generalized Process Capability Index (GPCI)
chains using Importance Sampling (Sampling Importance Resampling, SIR) for uncensored data.
}
\examples{
data_x <- rnorm(50, mean = 10, sd = 1)
dist_norm <- dist_normal()
fit_imp <- impsam_gpci(
  data = data_x,
  distribution = dist_norm,
  chain_length = 500,
  burn_in = 100,
  thinning = 1,
  USL = 13, LSL = 7
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{summary.gpciImpSam}
\alias{summary.gpciImpSam}
\title{Summary and Diagnostics for gpciImpSam Objects}
\usage{
\method{summary}{gpciImpSam}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"gpciImpSam"}.}

\item{...}{Additional arguments.}
}
\value{
A data frame containing diagnostic metrics for each evaluated GPCI.
}
\description{
Computes estimated value, bias, MSE, Risk value, HPD intervals at 90\%, 95\%, and 99\% significance levels,
Heidelberger and Welch MCMC convergence diagnostic, and convergence probability.
}
\examples{
data_x <- rnorm(40, mean = 10, sd = 1)
fit_imp <- impsam_gpci(data = data_x, distribution = dist_normal(), USL = 13, LSL = 7)
summary(fit_imp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{gpci_impsam}
\alias{gpci_impsam}
\title{High-Level User Interface Function for Importance Sampling GPCIs}
\usage{
gpci_impsam(
  data,
  pdf = NULL,
  cdf = NULL,
  sf = NULL,
  prior = NULL,
  chain_length = 1000,
  burn_in = 200,
  thinning = 1,
  USL,
  LSL,
  target = (USL + LSL)/2,
  indices = NULL
)
}
\arguments{
\item{data}{Numeric vector of process observations.}

\item{pdf}{Custom PDF/PMF function \code{function(x, ...)}.}

\item{cdf}{Custom CDF function \code{function(x, ...)}.}

\item{sf}{Custom Survival Function \code{function(x, ...)}.}

\item{prior}{Prior distribution hyperparameters list or log-prior function.}

\item{chain_length}{MCMC chain length (default 1000).}

\item{burn_in}{MCMC burn-in iterations (default 200).}

\item{thinning}{Thinning interval (default 1).}

\item{USL}{Upper Specification Limit.}

\item{LSL}{Lower Specification Limit.}

\item{target}{Target process value.}

\item{indices}{GPCI names to evaluate.}
}
\value{
A list of class \code{"gpciImpSam"} with full summary results.
}
\description{
Main user interface to fit uncensored data under Importance Sampling and evaluate GPCIs,
returning point estimates, parameter chains, GPCI chains, HPD intervals, and convergence diagnostics.
}
\examples{
gpci_impsam(
  data = rnorm(50, 10, 1),
  pdf = function(x, mean = 0, sd = 1) dnorm(x, mean, sd),
  cdf = function(x, mean = 0, sd = 1) pnorm(x, mean, sd),
  chain_length = 300, burn_in = 50,
  USL = 13, LSL = 7
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{safe_integrate}
\alias{safe_integrate}
\title{Safe Numerical Integration Helper}
\usage{
safe_integrate(f, lower, upper, ...)
}
\arguments{
\item{f}{Function to integrate.}

\item{lower}{Lower limit of integration.}

\item{upper}{Upper limit of integration.}

\item{...}{Additional arguments passed to \code{f}.}
}
\value{
Numeric scalar representing the integral value.
}
\description{
Evaluates definite integral of a 1D function safely with error handling and fallback bounds.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{dist_logistic_exponential}
\alias{dist_logistic_exponential}
\title{Pre-defined Logistic-Exponential Distribution}
\usage{
dist_logistic_exponential(alpha = 1, lambda = 1)
}
\arguments{
\item{alpha}{Scale/shape parameter (default is 1).}

\item{lambda}{Scale/shape parameter (default is 1).}
}
\value{
A \code{gpci_dist} object.
}
\description{
Pre-defined Logistic-Exponential Distribution
}
\examples{
d_le <- dist_logistic_exponential(alpha = 1, lambda = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{dist_gamma}
\alias{dist_gamma}
\title{Standard Pre-defined Gamma Distribution}
\usage{
dist_gamma(shape = 1, rate = 1)
}
\arguments{
\item{shape}{Shape parameter (default is 1).}

\item{rate}{Rate parameter (default is 1).}
}
\value{
A \code{gpci_dist} object.
}
\description{
Standard Pre-defined Gamma Distribution
}
\examples{
d_gam <- dist_gamma(shape = 2, rate = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indices.R
\name{capability}
\alias{capability}
\title{Compute Process Capability Indices (GPCIs)}
\usage{
capability(
  data = NULL,
  distribution,
  USL,
  LSL,
  target = (USL + LSL)/2,
  indices = NULL,
  u = 1,
  v = 1,
  mode = c("moments", "quantile"),
  fit = TRUE,
  fit_method = "mle",
  C0 = 1,
  C1 = 0,
  C2 = 1,
  tolerance_t = USL - LSL,
  P0 = 0.9973002,
  LDL = LSL,
  UDL = USL
)
}
\arguments{
\item{data}{Numeric vector of observed process data (optional if distribution is supplied with parameters).}

\item{distribution}{A \code{gpci_dist} object.}

\item{USL}{Upper Specification Limit.}

\item{LSL}{Lower Specification Limit.}

\item{target}{Target value of process (defaults to midpoint of USL and LSL).}

\item{indices}{Character vector of index names to evaluate. Default computes all GPCIs:
\code{c("Cpy", "Cp", "Cpk", "Cpu", "Cpl", "Cpm", "Cpmk", "CpTk", "Spmk", "Cpc", "CNpk", "CNpmc", "CNpmkc", "Cp_uv")}.}

\item{u}{Parameter \code{u} for Vännman's \code{Cp(u,v)} family (default 1).}

\item{v}{Parameter \code{v} for Vännman's \code{Cp(u,v)} family (default 1).}

\item{mode}{Mode of computation: \code{"moments"} (mean/sd) or \code{"quantile"} (quantiles).}

\item{fit}{Logical. If \code{TRUE} (default) and data is supplied, parameters are estimated from data.}

\item{fit_method}{Parameter estimation method (\code{"mle"}).}

\item{C0, C1, C2}{Parameters for tolerance loss function (defaults: 1, 0, 1).}

\item{tolerance_t}{Process tolerance span (defaults to \code{USL - LSL}).}

\item{P0}{Expected yield baseline (default 0.9973002).}

\item{LDL, UDL}{Lower and upper desired limits.}
}
\value{
A named numeric vector of evaluated GPCI values.
}
\description{
Computes classical and generalized Process Capability Indices for a given dataset or
process distribution parameters under uncensored data.
}
\examples{
dist_n <- dist_normal(mean = 10, sd = 1.5)
capability(
  distribution = dist_n,
  USL = 15, LSL = 5, target = 10,
  indices = c("Cpy", "Cp", "Cpk", "Cpm", "Cpmk")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{heidelberger_welch}
\alias{heidelberger_welch}
\title{Heidelberger and Welch's MCMC Convergence Diagnostic}
\usage{
heidelberger_welch(x, alpha = 0.05, eps = 0.1)
}
\arguments{
\item{x}{Numeric vector representing an MCMC chain.}

\item{alpha}{Significance level for the test (default is 0.05).}

\item{eps}{Target maximum ratio of half-width to sample mean (default is 0.1).}
}
\value{
A list containing Cramér-von Mises statistic (\code{stat}), p-value (\code{pvalue}),
  stationarity test status (\code{passed}), half-width test status (\code{hw_passed}),
  half-width statistic (\code{hw_stat}), and convergence probability (\code{convergence_prob}).
}
\description{
Conducts stationarity and relative half-width tests under Heidelberger and Welch's
MCMC convergence diagnostic, returning test statistics, status, and convergence probability.
}
\examples{
samples <- rnorm(1000)
heidelberger_welch(samples)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{dist_exponential}
\alias{dist_exponential}
\title{Standard Pre-defined Exponential Distribution}
\usage{
dist_exponential(rate = 1)
}
\arguments{
\item{rate}{Rate parameter (default is 1).}
}
\value{
A \code{gpci_dist} object.
}
\description{
Standard Pre-defined Exponential Distribution
}
\examples{
d_exp <- dist_exponential(rate = 0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{safe_uniroot}
\alias{safe_uniroot}
\title{Safe Root Finder Helper}
\usage{
safe_uniroot(f, interval, ...)
}
\arguments{
\item{f}{Objective function.}

\item{interval}{Numeric vector of length 2 defining search range.}

\item{...}{Additional arguments passed to \code{f}.}
}
\value{
Numeric scalar solution.
}
\description{
Solves 1D non-linear equation f(x) = 0 safely within given interval.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimation.R
\name{fit_distribution}
\alias{fit_distribution}
\title{Uncensored Data Parameter Estimation via Maximum Likelihood}
\usage{
fit_distribution(data, distribution, method = "mle", initial = NULL)
}
\arguments{
\item{data}{Numeric vector of observed process observations.}

\item{distribution}{A \code{gpci_dist} object.}

\item{method}{Optimization method (default \code{"mle"}).}

\item{initial}{Named numeric vector or list of initial parameter values. If \code{NULL},
sensible defaults or sample moments are used.}
}
\value{
An updated \code{gpci_dist} object with estimated parameters.
}
\description{
Fits process distribution parameters to observed uncensored numeric data using
Maximum Likelihood Estimation (MLE).
}
\examples{
data_sample <- rnorm(100, mean = 10, sd = 2)
dist_fitted <- fit_distribution(data_sample, dist_normal())
dist_fitted$params
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution.R
\name{dist_normal}
\alias{dist_normal}
\title{Standard Pre-defined Normal Distribution}
\usage{
dist_normal(mean = 0, sd = 1)
}
\arguments{
\item{mean}{Distribution mean (default is 0).}

\item{sd}{Distribution standard deviation (default is 1).}
}
\value{
A \code{gpci_dist} object.
}
\description{
Standard Pre-defined Normal Distribution
}
\examples{
d_norm <- dist_normal(mean = 10, sd = 2)
}
